<?php

return [
    'attributes' => [
        'features' => 'OS Features',
        'hardware' => '硬體',
        'icon' => '圖示',
        'location' => '位置',
        'os' => '作業系統',
        'serial' => '序號',
        'sysName' => '系統名稱',
        'version' => '作業系統版本',
    ],
];
