<?php

return [

    /*
    |--------------------------------------------------------------------------
    | Password Reset Language Lines
    |--------------------------------------------------------------------------
    |
    | The following language lines are the default lines which match reasons
    | that are given by the password broker for a password update attempt
    | has failed, such as for an invalid token or invalid new password.
    |
    */

    'password' => 'Le mot de passe doit comporter au moins six caractères et doit être le même que la confirmation.',
    'reset' => 'Votre mot de passe a été réinitialisé !',
    'sent' => 'Nous vous avons envoyé un lien pour mettre à jour votre mot de passe',
    'token' => 'Le jeton de mise à jour du mot de passe est valide.',
    'user' => "Nous ne trouvons pas d'utilisateur avec cette adresse e-mail.",

];
