UPDATE `sensors` SET `sensor_oid` = REPLACE(`sensor_oid`, '1.3.6.1.', '.1.3.6.1.') WHERE `sensor_oid` LIKE '1.3.6.1.%';
UPDATE `sensors` SET `sensor_oid` = REPLACE(`sensor_oid`, 'GAMATRONIC-MIB::gamatronicLTD', '.1.3.6.1.4.1.6050') WHERE `sensor_oid` LIKE 'GAMATRONIC-MIB::gamatronicLTD%';
UPDATE `sensors` SET `sensor_oid` = REPLACE(`sensor_oid`, 'NMS-IF-MIB::curr', '.1.3.6.1.4.1.11606.10.9.63.1.7.1.6') WHERE `sensor_oid` LIKE 'NMS-IF-MIB::curr%';
UPDATE `sensors` SET `sensor_oid` = REPLACE(`sensor_oid`, 'NMS-IF-MIB::rxPower', '.1.3.6.1.4.1.11606.10.9.63.1.7.1.3') WHERE `sensor_oid` LIKE 'NMS-IF-MIB::rxPower%';
UPDATE `sensors` SET `sensor_oid` = REPLACE(`sensor_oid`, 'NMS-IF-MIB::txPower', '.1.3.6.1.4.1.11606.10.9.63.1.7.1.2') WHERE `sensor_oid` LIKE 'NMS-IF-MIB::txPower%';
UPDATE `sensors` SET `sensor_oid` = REPLACE(`sensor_oid`, 'ifSfpParameterTable', '.1.3.6.1.4.1.11606.10.9.63.1.7') WHERE `sensor_oid` LIKE 'ifSfpParameterTable%';
UPDATE `sensors` SET `sensor_oid` = REPLACE(`sensor_oid`, 'PowerNet-MIB::upsAdvBatteryRunTimeRemaining', '.1.3.6.1.4.1.318.1.1.1.2.2.3') WHERE `sensor_oid` LIKE 'PowerNet-MIB::upsAdvBatteryRunTimeRemaining%';
UPDATE `sensors` SET `sensor_oid` = REPLACE(`sensor_oid`, 'receivePower', '.1.3.6.1.4.1.17713.7.12.2') WHERE `sensor_oid` LIKE 'receivePower%';
UPDATE `sensors` SET `sensor_oid` = REPLACE(`sensor_oid`, 'cambiumSTADLRSSI', '.1.3.6.1.4.1.17713.21.1.2.3') WHERE `sensor_oid` LIKE 'cambiumSTADLRSSI%';
UPDATE `sensors` SET `sensor_oid` = REPLACE(`sensor_oid`, 'lastPowerLevel', '.1.3.6.1.4.1.17713.21.1.2.3') WHERE `sensor_oid` LIKE 'lastPowerLevel%';
UPDATE `sensors` SET `sensor_oid` = REPLACE(`sensor_oid`, 'radioDbmInt', '.1.3.6.1.4.1.161.19.3.2.2.21') WHERE `sensor_oid` LIKE 'radioDbmInt%';
UPDATE `sensors` SET `sensor_oid` = REPLACE(`sensor_oid`, 'airIRRCUnitStatusRackInletTempMetric', '.1.3.6.1.4.1.318.1.1.13.3.2.2.2.7') WHERE `sensor_oid` LIKE 'airIRRCUnitStatusRackInletTempMetric%';
UPDATE `sensors` SET `sensor_oid` = REPLACE(`sensor_oid`, 'airIRRCUnitStatusSupplyAirTempMetric', '.1.3.6.1.4.1.318.1.1.13.3.2.2.2.9') WHERE `sensor_oid` LIKE 'airIRRCUnitStatusSupplyAirTempMetric%';
UPDATE `sensors` SET `sensor_oid` = REPLACE(`sensor_oid`, 'airIRRCUnitStatusReturnAirTempMetric', '.1.3.6.1.4.1.318.1.1.13.3.2.2.2.11') WHERE `sensor_oid` LIKE 'airIRRCUnitStatusReturnAirTempMetric%';
UPDATE `sensors` SET `sensor_oid` = REPLACE(`sensor_oid`, 'airIRRCUnitStatusEnteringFluidTemperatureMetric', '.1.3.6.1.4.1.318.1.1.13.3.2.2.2.24') WHERE `sensor_oid` LIKE 'airIRRCUnitStatusEnteringFluidTemperatureMetric%';
UPDATE `sensors` SET `sensor_oid` = REPLACE(`sensor_oid`, 'airIRRCUnitStatusLeavingFluidTemperatureMetric', '.1.3.6.1.4.1.318.1.1.13.3.2.2.2.26') WHERE `sensor_oid` LIKE 'airIRRCUnitStatusLeavingFluidTemperatureMetric%';
UPDATE `sensors` SET `sensor_oid` = REPLACE(`sensor_oid`, 'boxTemperatureC', '.1.3.6.1.4.1.161.19.3.3.1.35') WHERE `sensor_oid` LIKE 'boxTemperatureC%';
UPDATE `sensors` SET `sensor_oid` = REPLACE(`sensor_oid`, 'NMS-IF-MIB::temperature', '.1.3.6.1.4.1.11606.10.9.63.1.7.1.44') WHERE `sensor_oid` LIKE 'NMS-IF-MIB::temperature%';
UPDATE `sensors` SET `sensor_oid` = REPLACE(`sensor_oid`, 'systemTemperature', '.1.3.6.1.4.1.17163.1.1.2.9') WHERE `sensor_oid` LIKE 'systemTemperature%';
UPDATE `sensors` SET `sensor_oid` = REPLACE(`sensor_oid`, 'rbSysTemperature', '.1.3.6.1.4.1.31926.1.2') WHERE `sensor_oid` LIKE 'rbSysTemperature%';
