<?php

return [

    /*
    |--------------------------------------------------------------------------
    | Sensors Language Lines
    |--------------------------------------------------------------------------
    |
    | The following language lines are used to translate names and units of sensors
    |
    */

    'airflow' => [
        'short' => "Flux d'air",
        'long' => "Flux d'air",
        'unit' => 'cfm',
        'unit_long' => 'mètre cube par minute',
    ],
    'ber' => [
        'short' => 'BER',
        'long' => "Taux d'erreurs",
        'unit' => '',
        'unit_long' => '',
    ],
    'charge' => [
        'short' => 'Charge',
        'long' => 'Pourcentage de charge',
        'unit' => '%',
        'unit_long' => 'Pourcent',
    ],
    'chromatic_dispersion' => [
        'short' => 'Dispersion chromatique',
        'long' => 'Dispersion chromatique',
        'unit' => 'ps/nm/km',
        'unit_long' => 'Picosecondes par nanomètre par kilomètre',
    ],
    'cooling' => [
        'short' => 'Refroidissement',
        'long' => '',
        'unit' => 'W',
        'unit_long' => 'Watts',
    ],
    'count' => [
        'short' => 'Count',
        'long' => 'Count',
        'unit' => '',
        'unit_long' => '',
    ],
    'current' => [
        'short' => 'Courant',
        'long' => 'Courant',
        'unit' => 'A',
        'unit_long' => 'Ampères',
    ],
    'dbm' => [
        'short' => 'dBm',
        'long' => 'dBm',
        'unit' => 'dBm',
        'unit_long' => 'Décibel-Milliwatts',
    ],
    'delay' => [
        'short' => 'Delay',
        'long' => 'Delay',
        'unit' => 's',
        'unit_long' => 'Seconds',
    ],
    'eer' => [
        'short' => 'EER',
        'long' => "Ratio d'efficacité énergétique",
        'unit' => '',
        'unit_long' => '',
    ],
    'fanspeed' => [
        'short' => 'Vitesse du ventilateur',
        'long' => 'Vitesse du ventilateur',
        'unit' => 'RPM',
        'unit_long' => 'Rotations par Minute',
    ],
    'frequency' => [
        'short' => 'Fréquence',
        'long' => 'Fréquence',
        'unit' => 'Hz',
        'unit_long' => 'Hertz',
    ],
    'humidity' => [
        'short' => 'Humidité',
        'long' => "Pourcentage d'humidité",
        'unit' => '%',
        'unit_long' => 'Pourcent',
    ],
    'load' => [
        'short' => 'Charge',
        'long' => 'Pourcentage de charge',
        'unit' => '%',
        'unit_long' => 'Pourcent',
    ],
    'power' => [
        'short' => 'Puissance',
        'long' => 'Puissance',
        'unit' => 'W',
        'unit_long' => 'Watts',
    ],
    'power_consumed' => [
        'short' => 'Puissance consommée',
        'long' => 'Puissance consommée',
        'unit' => 'kWh',
        'unit_long' => 'Kilowatt-heure',
    ],
    'power_factor' => [
        'short' => 'Facteur de puissance',
        'long' => 'Facteur de puissance',
        'unit' => '',
        'unit_long' => '',
    ],
    'pressure' => [
        'short' => 'Pression',
        'long' => 'Pression',
        'unit' => 'kPa',
        'unit_long' => 'Kilopascal',
    ],
    'quality_factor' => [
        'short' => 'Facteur de qualité',
        'long' => 'Facteur de qualité',
        'unit' => '',
        'unit_long' => '',
    ],
    'runtime' => [
        'short' => "Temps d'éxécution",
        'long' => "Temps d'éxécution",
        'unit' => 'Min',
        'unit_long' => 'Minutes',
    ],
    'signal' => [
        'short' => 'Signal',
        'long' => 'Signal',
        'unit' => 'dBm',
        'unit_long' => 'Decibel-milliwatt',
    ],
    'snr' => [
        'short' => 'SNR',
        'long' => 'Rapport signal sur bruit',
        'unit' => 'dB',
        'unit_long' => 'Decibel',
    ],
    'state' => [
        'short' => 'Etat',
        'long' => 'Etat',
        'unit' => '',
    ],
    'temperature' => [
        'short' => 'Température',
        'long' => 'Température',
        'unit' => '°C',
        'unit_long' => '° Celsius',
    ],
    'voltage' => [
        'short' => 'Voltage',
        'long' => 'voltage',
        'unit' => 'V',
        'unit_long' => 'Volts',
    ],
    'waterflow' => [
        'short' => "Ecoulement de l'eau",
        'long' => "Ecoulement de l'eau",
        'unit' => 'l/m',
        'unit_long' => 'Litres par minute',
    ],
];
