<?php

return [
    'attributes' => [
        'features' => 'OS Features',
        'hardware' => 'Hardware',
        'icon' => 'Icon',
        'location' => 'Location',
        'os' => 'Device OS',
        'serial' => 'Serial',
        'sysName' => 'sysName',
        'version' => 'OS Version',
        'type' => 'Device type',
    ],
];
