<?php

return [
    'alerts' => [
        'title' => 'Alerts',
    ],
    'alertlog' => [
        'title' => 'Alert History',
    ],
    'alertlog-stats' => [
        'title' => 'Alert History Stats',
    ],
    'availability-map' => [
        'title' => 'Availability Map',
    ],
    'component-status' => [
        'title' => 'Component Status',
    ],
    'device-summary-horiz' => [
        'title' => 'Device Summary Horizontal',
    ],
    'device-summary-vert' => [
        'title' => 'Device Summary Vertical',
    ],
    'device-types' => [
        'title' => 'Device Types',
    ],
    'eventlog' => [
        'title' => 'Eventlog',
    ],
    'generic-graph' => [
        'title' => 'Graph',
    ],
    'generic-image' => [
        'title' => 'External Images',
    ],
    'globe' => [
        'title' => 'Globe Map',
    ],
    'graylog' => [
        'title' => 'Graylog',
    ],
    'notes' => [
        'title' => 'Notes',
    ],
    'server-stats' => [
        'title' => 'Server Stats',
    ],
    'syslog' => [
        'title' => 'Syslog',
    ],
    'top-devices' => [
        'title' => 'Top Devices',
    ],
    'top-errors' => [
        'title' => 'Top Errors',
    ],
    'top-interfaces' => [
        'title' => 'Top Interfaces',
    ],
    'worldmap' => [
        'title' => 'World Map',
    ],
];
