<?php

/*
 * This file is part of Psy Shell.
 *
 * (c) 2012-2022 Justin Hileman
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Psy\Formatter;

/**
 * Formatter interface.
 *
 * @deprecated this interface only exists for backwards compatibility. Use ReflectorFormatter.
 */
interface Formatter extends ReflectorFormatter
{
}
