<?php

// autoload_namespaces.php @generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname($vendorDir);

return array(
    'Net_DNS2' => array($vendorDir . '/pear/net_dns2'),
    'HTMLPurifier' => array($vendorDir . '/ezyang/htmlpurifier/library'),
    'Crypt_CHAP_' => array($vendorDir . '/dapphp/radius/lib'),
    'Console_Color2' => array($vendorDir . '/pear/console_color2'),
);
