export default {
    "en": {
        "help": {
            "yes": "yes",
            "no": "no"
        }
    },
    "sv": {
        "help": {
            "yes": "ja",
            "no": "nej"
        }
    }
}
