ALTER TABLE  `bgpPeers_cbgp` ADD  `AcceptedPrefixes_delta` INT NOT NULL ,ADD  `AcceptedPrefixes_prev` INT NOT NULL ,ADD  `DeniedPrefixes_delta` INT NOT NULL ,ADD  `DeniedPrefixes_prev` INT NOT NULL ,ADD  `AdvertisedPrefixes_delta` INT NOT NULL ,ADD  `AdvertisedPrefixes_prev` INT NOT NULL ,ADD  `SuppressedPrefixes_delta` INT NOT NULL ,ADD  `SuppressedPrefixes_prev` INT NOT NULL ,ADD  `WithdrawnPrefixes_delta` INT NOT NULL ,ADD  `WithdrawnPrefixes_prev` INT NOT NULL ;
