INSERT INTO `alert_templates` (name, template) values('Default Alert Template', '{{ $alert->title }}\nSeverity: {{ $alert->severity }}\n @if ($alert->state == 0) Time elapsed: {{ $alert->elapsed }}\n @endif Timestamp: {{ $alert->timestamp }}\nUnique-ID: {{ $alert->uid }}\nRule:  @if ($alert->name) {{ $alert->name }} @else {{ $alert->rule }} @endif\n @if ($alert->faults) Faults:\n @foreach ($alert->faults as $key => $value)  #{{ $key }}: {{ $value[\'string\'] }}\n @endforeach  @endif Alert sent to:  @foreach ($alert->contacts as $key => $value){{ $value }} <{{ $key }}>  @endforeach');
