<?php

return [

    /*
    |--------------------------------------------------------------------------
    | Password Reset Language Lines
    |--------------------------------------------------------------------------
    |
    | The following language lines are the default lines which match reasons
    | that are given by the password broker for a password update attempt
    | has failed, such as for an invalid token or invalid new password.
    |
    */

    'password' => '密碼至少需要六個字元，並且要確認兩者相符。',
    'reset' => '您的密碼已重置。',
    'sent' => '已經寄送密碼重置連結至您的電子郵件信箱。',
    'token' => '此密碼重置權仗無效。',
    'user' => '找不到使用者的電子郵件位址。',

];
