<?php

/**
 * This file is part of the Carbon package.
 *
 * (c) Brian Nesbitt <brian@nesbot.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

/*
 * Authors:
 * - sheriffmarley
 * - Timo
 * - Michael Hohl
 * - Namoshek
 * - Bernhard Baumrock (BernhardBaumrock)
 */
return array_replace_recursive(require __DIR__.'/de.php', [
    'months' => [
        0 => 'Jänner',
    ],
    'months_short' => [
        0 => 'Jän',
    ],
]);
