CREATE TABLE IF NOT EXISTS `slas` (  `sla_id` int(11) NOT NULL auto_increment,  `device_id` int(11) NOT NULL,  `sla_nr` int(11) NOT NULL,  `owner` varchar(255) NOT NULL,  `tag` varchar(255) NOT NULL, `rtt_type` varchar(16) NOT NULL, `status` tinyint(1) NOT NULL, `deleted` tinyint(1) NOT NULL DEFAULT '0', PRIMARY KEY  (`sla_id`),  UNIQUE KEY `unique_key` (`device_id`,`sla_nr`),  KEY `device_id` (`device_id`)) ENGINE=MyISAM  CHARSET=utf8 COLLATE=utf8_bin;
