INSERT INTO `config` (config_name,config_value,config_default,config_descr,config_group,config_group_order,config_sub_group,config_sub_group_order,config_hidden,config_disabled) values('alert.transports.jira.prjkey','','','Jira Project Key','alerting',0,'transports',0,default,default);
INSERT INTO `config` (config_name,config_value,config_default,config_descr,config_group,config_group_order,config_sub_group,config_sub_group_order,config_hidden,config_disabled) values('alert.transports.jira.url','','','Jira URL','alerting',0,'transports',0,default,default);
INSERT INTO `config` (config_name,config_value,config_default,config_descr,config_group,config_group_order,config_sub_group,config_sub_group_order,config_hidden,config_disabled) values('alert.transports.jira.username','','','Jira Username','alerting',0,'transports',0,default,default);
INSERT INTO `config` (config_name,config_value,config_default,config_descr,config_group,config_group_order,config_sub_group,config_sub_group_order,config_hidden,config_disabled) values('alert.transports.jira.password','','','Jira Password','alerting',0,'transports',0,default,default);
INSERT INTO `config` (config_name,config_value,config_default,config_descr,config_group,config_group_order,config_sub_group,config_sub_group_order,config_hidden,config_disabled) values('alert.transports.jira.issuetype','','','Jira Issue Type','alerting',0,'transports',0,default,default);
