CREATE TABLE IF NOT EXISTS `access_points` (  `accesspoint_id` int(11) NOT NULL AUTO_INCREMENT,  `device_id` int(11) NOT NULL,  `name` varchar(255) NOT NULL,  `radio_number` tinyint(4) DEFAULT NULL,  `type` varchar(16) NOT NULL,  `mac_addr` varchar(24) NOT NULL,  `deleted` tinyint(1) NOT NULL DEFAULT '0',  `channel` tinyint(4) unsigned NOT NULL DEFAULT '0',  `txpow` tinyint(4) NOT NULL DEFAULT '0',  `radioutil` tinyint(4) NOT NULL DEFAULT '0',  `numasoclients` smallint(6) NOT NULL DEFAULT '0',  `nummonclients` smallint(6) NOT NULL DEFAULT '0',  `numactbssid` tinyint(4) NOT NULL DEFAULT '0',  `nummonbssid` tinyint(4) NOT NULL DEFAULT '0',  `interference` tinyint(3) unsigned NOT NULL,  PRIMARY KEY (`accesspoint_id`),  KEY `deleted` (`deleted`),  KEY `name` (`name`,`radio_number`)) ENGINE=InnoDB  DEFAULT CHARSET=utf8 COMMENT='Access Points';