<?php

$graph_type = 'sensor_quality_factor';
$sensor_class = 'quality_factor';
$sensor_unit = 'dB';
$sensor_type = 'quality_factor';

require 'includes/html/pages/device/overview/generic/sensor.inc.php';
