<?php

/**
 * Rules for Nette Coding Standard
 * https://github.com/nette/coding-standard
 */

declare(strict_types=1);

return [
	// use function in Arrays.php, Callback.php, Html.php, Strings.php
	'single_import_per_statement' => false,
	'ordered_imports' => false,
];
