<?php

return [

    /*
    |--------------------------------------------------------------------------
    | Password Reset Language Lines
    |--------------------------------------------------------------------------
    |
    | The following language lines are the default lines which match reasons
    | that are given by the password broker for a password update attempt
    | has failed, such as for an invalid token or invalid new password.
    |
    */

    'reset' => 'Vaša lozinka je resetovana!',
    'sent' => 'Poslali smo vam link za reset loinke u e-mailu!',
    'throttled' => 'Molim sačekajte pre nego pokušate ponovo.',
    'token' => 'Vaučer za reset lozinke je neispravan.',
    'user' => 'Ne postoji korisnik sa takvom e-mail adresom',

];
