<?php

return [
    'notification-subscription-status' => [
        'no-support' => 'Questo browser non supporta le notifiche',
        'no-transport' => 'Per abilitarle, deve esserci una modalità di avviso che fa riferimento a questo utente',
        'enabled' => 'Notifiche abilitate per questo browser',
        'disabled' => 'Notifiche disabilitate per questo browser',
        'enable' => 'Abilitato',
        'disable' => 'Disabilitato',
    ],
];
