<?php

return [

    /*
    |--------------------------------------------------------------------------
    | Password Reset Language Lines
    |--------------------------------------------------------------------------
    |
    | The following language lines are the default lines which match reasons
    | that are given by the password broker for a password update attempt
    | has failed, such as for an invalid token or invalid new password.
    |
    */

    'reset' => 'La tua password è stata reimpostata!',
    'sent' => 'Ti abbiamo inviato per e-mail il link per reimpostare la password!',
    'throttled' => 'Per favore attendi prima di riprovare.',
    'token' => 'Questo token per reimpostare la password non è valido.',
    'user' => "Non è stato possibile trovare un utente con quell'indirizzo e-mail.",

];
