<?php return array(
    'root' => array(
        'name' => 'librenms/librenms',
        'pretty_version' => '1.0.0+no-version-set',
        'version' => '1.0.0.0',
        'reference' => NULL,
        'type' => 'project',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'dev' => false,
    ),
    'versions' => array(
        'amenadiel/jpgraph' => array(
            'pretty_version' => 'v4.1.1',
            'version' => '4.1.1.0',
            'reference' => 'e196d51a520a4182cc26240b4a0744495706f11a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../amenadiel/jpgraph',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'asm89/stack-cors' => array(
            'pretty_version' => 'v2.1.1',
            'version' => '2.1.1.0',
            'reference' => '73e5b88775c64ccc0b84fb60836b30dc9d92ac4a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../asm89/stack-cors',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'brick/math' => array(
            'pretty_version' => '0.10.2',
            'version' => '0.10.2.0',
            'reference' => '459f2781e1a08d52ee56b0b1444086e038561e3f',
            'type' => 'library',
            'install_path' => __DIR__ . '/../brick/math',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/socket-raw' => array(
            'pretty_version' => 'v1.6.0',
            'version' => '1.6.0.0',
            'reference' => '91e9f619f6769f931454a9882c21ffd7623d06cb',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/socket-raw',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'dapphp/radius' => array(
            'pretty_version' => '2.5.8',
            'version' => '2.5.8.0',
            'reference' => '7b2533f9b89b134cb92b6a260cc2e54b347390f9',
            'type' => 'library',
            'install_path' => __DIR__ . '/../dapphp/radius',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'dflydev/dot-access-data' => array(
            'pretty_version' => 'v3.0.2',
            'version' => '3.0.2.0',
            'reference' => 'f41715465d65213d644d3141a6a93081be5d3549',
            'type' => 'library',
            'install_path' => __DIR__ . '/../dflydev/dot-access-data',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'doctrine/cache' => array(
            'pretty_version' => '2.2.0',
            'version' => '2.2.0.0',
            'reference' => '1ca8f21980e770095a31456042471a57bc4c68fb',
            'type' => 'library',
            'install_path' => __DIR__ . '/../doctrine/cache',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'doctrine/dbal' => array(
            'pretty_version' => '2.13.9',
            'version' => '2.13.9.0',
            'reference' => 'c480849ca3ad6706a39c970cdfe6888fa8a058b8',
            'type' => 'library',
            'install_path' => __DIR__ . '/../doctrine/dbal',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'doctrine/deprecations' => array(
            'pretty_version' => 'v1.0.0',
            'version' => '1.0.0.0',
            'reference' => '0e2a4f1f8cdfc7a92ec3b01c9334898c806b30de',
            'type' => 'library',
            'install_path' => __DIR__ . '/../doctrine/deprecations',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'doctrine/event-manager' => array(
            'pretty_version' => '1.2.0',
            'version' => '1.2.0.0',
            'reference' => '95aa4cb529f1e96576f3fda9f5705ada4056a520',
            'type' => 'library',
            'install_path' => __DIR__ . '/../doctrine/event-manager',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'doctrine/inflector' => array(
            'pretty_version' => '2.0.6',
            'version' => '2.0.6.0',
            'reference' => 'd9d313a36c872fd6ee06d9a6cbcf713eaa40f024',
            'type' => 'library',
            'install_path' => __DIR__ . '/../doctrine/inflector',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'doctrine/lexer' => array(
            'pretty_version' => '1.2.3',
            'version' => '1.2.3.0',
            'reference' => 'c268e882d4dbdd85e36e4ad69e02dc284f89d229',
            'type' => 'library',
            'install_path' => __DIR__ . '/../doctrine/lexer',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'dragonmantank/cron-expression' => array(
            'pretty_version' => 'v3.3.2',
            'version' => '3.3.2.0',
            'reference' => '782ca5968ab8b954773518e9e49a6f892a34b2a8',
            'type' => 'library',
            'install_path' => __DIR__ . '/../dragonmantank/cron-expression',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'easybook/geshi' => array(
            'pretty_version' => 'v1.0.8.19',
            'version' => '1.0.8.19',
            'reference' => 'b4df5fa84a44d4e12eff67263a701eac7e157241',
            'type' => 'library',
            'install_path' => __DIR__ . '/../easybook/geshi',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'egulias/email-validator' => array(
            'pretty_version' => '2.1.25',
            'version' => '2.1.25.0',
            'reference' => '0dbf5d78455d4d6a41d186da50adc1122ec066f4',
            'type' => 'library',
            'install_path' => __DIR__ . '/../egulias/email-validator',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ezyang/htmlpurifier' => array(
            'pretty_version' => 'v4.13.0',
            'version' => '4.13.0.0',
            'reference' => '08e27c97e4c6ed02f37c5b2b20488046c8d90d75',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ezyang/htmlpurifier',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'facade/flare-client-php' => array(
            'pretty_version' => '1.10.0',
            'version' => '1.10.0.0',
            'reference' => '213fa2c69e120bca4c51ba3e82ed1834ef3f41b8',
            'type' => 'library',
            'install_path' => __DIR__ . '/../facade/flare-client-php',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'facade/ignition' => array(
            'pretty_version' => '2.17.6',
            'version' => '2.17.6.0',
            'reference' => '6acd82e986a2ecee89e2e68adfc30a1936d1ab7c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../facade/ignition',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'facade/ignition-contracts' => array(
            'pretty_version' => '1.0.2',
            'version' => '1.0.2.0',
            'reference' => '3c921a1cdba35b68a7f0ccffc6dffc1995b18267',
            'type' => 'library',
            'install_path' => __DIR__ . '/../facade/ignition-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'fico7489/laravel-pivot' => array(
            'pretty_version' => '3.0.7',
            'version' => '3.0.7.0',
            'reference' => '97fbc8c7c65b652f21b0ff78a430712f3f14a515',
            'type' => 'library',
            'install_path' => __DIR__ . '/../fico7489/laravel-pivot',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'fruitcake/laravel-cors' => array(
            'pretty_version' => 'v2.2.0',
            'version' => '2.2.0.0',
            'reference' => '783a74f5e3431d7b9805be8afb60fd0a8f743534',
            'type' => 'library',
            'install_path' => __DIR__ . '/../fruitcake/laravel-cors',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'graham-campbell/result-type' => array(
            'pretty_version' => 'v1.1.0',
            'version' => '1.1.0.0',
            'reference' => 'a878d45c1914464426dc94da61c9e1d36ae262a8',
            'type' => 'library',
            'install_path' => __DIR__ . '/../graham-campbell/result-type',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'guzzlehttp/guzzle' => array(
            'pretty_version' => '7.5.0',
            'version' => '7.5.0.0',
            'reference' => 'b50a2a1251152e43f6a37f0fa053e730a67d25ba',
            'type' => 'library',
            'install_path' => __DIR__ . '/../guzzlehttp/guzzle',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'guzzlehttp/promises' => array(
            'pretty_version' => '1.5.2',
            'version' => '1.5.2.0',
            'reference' => 'b94b2807d85443f9719887892882d0329d1e2598',
            'type' => 'library',
            'install_path' => __DIR__ . '/../guzzlehttp/promises',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'guzzlehttp/psr7' => array(
            'pretty_version' => '2.4.3',
            'version' => '2.4.3.0',
            'reference' => '67c26b443f348a51926030c83481b85718457d3d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../guzzlehttp/psr7',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'illuminate/auth' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/broadcasting' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/bus' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/cache' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/collections' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/config' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/console' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/container' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/contracts' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/cookie' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/database' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/encryption' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/events' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/filesystem' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/hashing' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/http' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/log' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/macroable' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/mail' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/notifications' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/pagination' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/pipeline' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/queue' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/redis' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/routing' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/session' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/support' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/testing' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/translation' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/validation' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'illuminate/view' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v8.83.27',
            ),
        ),
        'influxdb/influxdb-php' => array(
            'pretty_version' => '1.15.2',
            'version' => '1.15.2.0',
            'reference' => 'd6e59f4f04ab9107574fda69c2cbe36671253d03',
            'type' => 'library',
            'install_path' => __DIR__ . '/../influxdb/influxdb-php',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'justinrainbow/json-schema' => array(
            'pretty_version' => '5.2.12',
            'version' => '5.2.12.0',
            'reference' => 'ad87d5a5ca981228e0e205c2bc7dfb8e24559b60',
            'type' => 'library',
            'install_path' => __DIR__ . '/../justinrainbow/json-schema',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'laravel-notification-channels/webpush' => array(
            'pretty_version' => '7.0.0',
            'version' => '7.0.0.0',
            'reference' => 'ed40628fd85492972c47c7796a7415c755258a58',
            'type' => 'library',
            'install_path' => __DIR__ . '/../laravel-notification-channels/webpush',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'laravel/framework' => array(
            'pretty_version' => 'v8.83.27',
            'version' => '8.83.27.0',
            'reference' => 'e1afe088b4ca613fb96dc57e6d8dbcb8cc2c6b49',
            'type' => 'library',
            'install_path' => __DIR__ . '/../laravel/framework',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'laravel/serializable-closure' => array(
            'pretty_version' => 'v1.2.2',
            'version' => '1.2.2.0',
            'reference' => '47afb7fae28ed29057fdca37e16a84f90cc62fae',
            'type' => 'library',
            'install_path' => __DIR__ . '/../laravel/serializable-closure',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'laravel/socialite' => array(
            'pretty_version' => 'v5.5.6',
            'version' => '5.5.6.0',
            'reference' => '1cd1682b709b8808a5b5dbb68179a58d1342aa7b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../laravel/socialite',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'laravel/tinker' => array(
            'pretty_version' => 'v2.7.3',
            'version' => '2.7.3.0',
            'reference' => '5062061b4924af3392225dd482ca7b4d85d8b8ef',
            'type' => 'library',
            'install_path' => __DIR__ . '/../laravel/tinker',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'laravel/ui' => array(
            'pretty_version' => 'v3.4.6',
            'version' => '3.4.6.0',
            'reference' => '65ec5c03f7fee2c8ecae785795b829a15be48c2c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../laravel/ui',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'league/commonmark' => array(
            'pretty_version' => '2.3.8',
            'version' => '2.3.8.0',
            'reference' => 'c493585c130544c4e91d2e0e131e6d35cb0cbc47',
            'type' => 'library',
            'install_path' => __DIR__ . '/../league/commonmark',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'league/config' => array(
            'pretty_version' => 'v1.2.0',
            'version' => '1.2.0.0',
            'reference' => '754b3604fb2984c71f4af4a9cbe7b57f346ec1f3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../league/config',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'league/flysystem' => array(
            'pretty_version' => '1.1.10',
            'version' => '1.1.10.0',
            'reference' => '3239285c825c152bcc315fe0e87d6b55f5972ed1',
            'type' => 'library',
            'install_path' => __DIR__ . '/../league/flysystem',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'league/mime-type-detection' => array(
            'pretty_version' => '1.11.0',
            'version' => '1.11.0.0',
            'reference' => 'ff6248ea87a9f116e78edd6002e39e5128a0d4dd',
            'type' => 'library',
            'install_path' => __DIR__ . '/../league/mime-type-detection',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'league/oauth1-client' => array(
            'pretty_version' => 'v1.10.1',
            'version' => '1.10.1.0',
            'reference' => 'd6365b901b5c287dd41f143033315e2f777e1167',
            'type' => 'library',
            'install_path' => __DIR__ . '/../league/oauth1-client',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'librenms/laravel-vue-i18n-generator' => array(
            'pretty_version' => '0.1.48',
            'version' => '0.1.48.0',
            'reference' => '956cb56130ee8c1faa48996184f5e37e9d25ac2f',
            'type' => 'library',
            'install_path' => __DIR__ . '/../librenms/laravel-vue-i18n-generator',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'librenms/librenms' => array(
            'pretty_version' => '1.0.0+no-version-set',
            'version' => '1.0.0.0',
            'reference' => NULL,
            'type' => 'project',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'mews/purifier' => array(
            'pretty_version' => '3.3.8',
            'version' => '3.3.8.0',
            'reference' => '164b420c4a80d2d0a89c1023f0aa9b3c0e3d6dc5',
            'type' => 'package',
            'install_path' => __DIR__ . '/../mews/purifier',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'minishlink/web-push' => array(
            'pretty_version' => 'v7.0.0',
            'version' => '7.0.0.0',
            'reference' => '8b023f2f5c6df846c0cfb87e13fb7ddf5e62c340',
            'type' => 'library',
            'install_path' => __DIR__ . '/../minishlink/web-push',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'monolog/monolog' => array(
            'pretty_version' => '2.8.0',
            'version' => '2.8.0.0',
            'reference' => '720488632c590286b88b80e62aa3d3d551ad4a50',
            'type' => 'library',
            'install_path' => __DIR__ . '/../monolog/monolog',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'mtdowling/cron-expression' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '^1.0',
            ),
        ),
        'nesbot/carbon' => array(
            'pretty_version' => '2.64.0',
            'version' => '2.64.0.0',
            'reference' => '889546413c97de2d05063b8cb7b193c2531ea211',
            'type' => 'library',
            'install_path' => __DIR__ . '/../nesbot/carbon',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'nette/schema' => array(
            'pretty_version' => 'v1.2.3',
            'version' => '1.2.3.0',
            'reference' => 'abbdbb70e0245d5f3bf77874cea1dfb0c930d06f',
            'type' => 'library',
            'install_path' => __DIR__ . '/../nette/schema',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'nette/utils' => array(
            'pretty_version' => 'v3.2.8',
            'version' => '3.2.8.0',
            'reference' => '02a54c4c872b99e4ec05c4aec54b5a06eb0f6368',
            'type' => 'library',
            'install_path' => __DIR__ . '/../nette/utils',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'nikic/php-parser' => array(
            'pretty_version' => 'v4.15.2',
            'version' => '4.15.2.0',
            'reference' => 'f59bbe44bf7d96f24f3e2b4ddc21cd52c1d2adbc',
            'type' => 'library',
            'install_path' => __DIR__ . '/../nikic/php-parser',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'nunomaduro/laravel-console-summary' => array(
            'pretty_version' => 'v1.8.0',
            'version' => '1.8.0.0',
            'reference' => '1b32af3f39a744223c4ed6d2a5080fc5baa037da',
            'type' => 'library',
            'install_path' => __DIR__ . '/../nunomaduro/laravel-console-summary',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'opis/closure' => array(
            'pretty_version' => '3.6.3',
            'version' => '3.6.3.0',
            'reference' => '3d81e4309d2a927abbe66df935f4bb60082805ad',
            'type' => 'library',
            'install_path' => __DIR__ . '/../opis/closure',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'paragonie/constant_time_encoding' => array(
            'pretty_version' => 'v2.6.3',
            'version' => '2.6.3.0',
            'reference' => '58c3f47f650c94ec05a151692652a868995d2938',
            'type' => 'library',
            'install_path' => __DIR__ . '/../paragonie/constant_time_encoding',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'paragonie/random_compat' => array(
            'pretty_version' => 'v9.99.100',
            'version' => '9.99.100.0',
            'reference' => '996434e5492cb4c3edcb9168db6fbb1359ef965a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../paragonie/random_compat',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'pear/console_color2' => array(
            'pretty_version' => '0.1.2',
            'version' => '0.1.2.0',
            'reference' => 'cab2b886aa55cc9e23464d1f49edf97bee9f5a63',
            'type' => 'library',
            'install_path' => __DIR__ . '/../pear/console_color2',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'pear/console_table' => array(
            'pretty_version' => 'v1.3.1',
            'version' => '1.3.1.0',
            'reference' => '1930c11897ca61fd24b95f2f785e99e0f36dcdea',
            'type' => 'library',
            'install_path' => __DIR__ . '/../pear/console_table',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'pear/net_dns2' => array(
            'pretty_version' => 'v1.5.3',
            'version' => '1.5.3.0',
            'reference' => 'dc8053772132a855b8bb6193422a959995f3a773',
            'type' => 'library',
            'install_path' => __DIR__ . '/../pear/net_dns2',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'php-amqplib/php-amqplib' => array(
            'pretty_version' => 'v3.4.0',
            'version' => '3.4.0.0',
            'reference' => '5c537cb724f2e181183c202e63f4303935344c5f',
            'type' => 'library',
            'install_path' => __DIR__ . '/../php-amqplib/php-amqplib',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'php-flasher/flasher' => array(
            'pretty_version' => 'v0.9.8',
            'version' => '0.9.8.0',
            'reference' => '07c043e4be6143fe8b4c59ffc8af7dd42ba04c4e',
            'type' => 'library',
            'install_path' => __DIR__ . '/../php-flasher/flasher',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'php-flasher/flasher-laravel' => array(
            'pretty_version' => 'v0.9.8',
            'version' => '0.9.8.0',
            'reference' => '247b28cfa9471ce6eef61673eb7574a5ba48ac91',
            'type' => 'library',
            'install_path' => __DIR__ . '/../php-flasher/flasher-laravel',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'phpmailer/phpmailer' => array(
            'pretty_version' => 'v6.7.1',
            'version' => '6.7.1.0',
            'reference' => '49cd7ea3d2563f028d7811f06864a53b1f15ff55',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phpmailer/phpmailer',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'phpoption/phpoption' => array(
            'pretty_version' => '1.9.0',
            'version' => '1.9.0.0',
            'reference' => 'dc5ff11e274a90cc1c743f66c9ad700ce50db9ab',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phpoption/phpoption',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'phpseclib/phpseclib' => array(
            'pretty_version' => '3.0.18',
            'version' => '3.0.18.0',
            'reference' => 'f28693d38ba21bb0d9f0c411ee5dae2b178201da',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phpseclib/phpseclib',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'predis/predis' => array(
            'pretty_version' => 'v1.1.10',
            'version' => '1.1.10.0',
            'reference' => 'a2fb02d738bedadcffdbb07efa3a5e7bd57f8d6e',
            'type' => 'library',
            'install_path' => __DIR__ . '/../predis/predis',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/container' => array(
            'pretty_version' => '1.1.2',
            'version' => '1.1.2.0',
            'reference' => '513e0666f7216c7459170d56df27dfcefe1689ea',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/container',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/container-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/event-dispatcher' => array(
            'pretty_version' => '1.0.0',
            'version' => '1.0.0.0',
            'reference' => 'dbefd12671e8a14ec7f180cab83036ed26714bb0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/event-dispatcher',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/event-dispatcher-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/http-client' => array(
            'pretty_version' => '1.0.1',
            'version' => '1.0.1.0',
            'reference' => '2dfb5f6c5eff0e91e20e913f8c5452ed95b86621',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-client',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-client-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/http-factory' => array(
            'pretty_version' => '1.0.1',
            'version' => '1.0.1.0',
            'reference' => '12ac7fcd07e5b077433f5f2bee95b3a771bf61be',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-factory',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-factory-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/http-message' => array(
            'pretty_version' => '1.0.1',
            'version' => '1.0.1.0',
            'reference' => 'f6561bf28d520154e4b0ec72be95418abe6d9363',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-message',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-message-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/log' => array(
            'pretty_version' => '2.0.0',
            'version' => '2.0.0.0',
            'reference' => 'ef29f6d262798707a9edd554e2b82517ef3a9376',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/log',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/log-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0|2.0',
                1 => '1.0.0 || 2.0.0 || 3.0.0',
            ),
        ),
        'psr/simple-cache' => array(
            'pretty_version' => '1.0.1',
            'version' => '1.0.1.0',
            'reference' => '408d5eafb83c57f6365a3ca330ff23aa4a5fa39b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/simple-cache',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/simple-cache-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psy/psysh' => array(
            'pretty_version' => 'v0.11.10',
            'version' => '0.11.10.0',
            'reference' => 'e9eadffbed9c9deb5426fd107faae0452bf20a36',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psy/psysh',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ralouphie/getallheaders' => array(
            'pretty_version' => '3.0.3',
            'version' => '3.0.3.0',
            'reference' => '120b605dfeb996808c31b6477290a714d356e822',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ralouphie/getallheaders',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ramsey/collection' => array(
            'pretty_version' => '1.3.0',
            'version' => '1.3.0.0',
            'reference' => 'ad7475d1c9e70b190ecffc58f2d989416af339b4',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ramsey/collection',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ramsey/uuid' => array(
            'pretty_version' => '4.7.0',
            'version' => '4.7.0.0',
            'reference' => '5ed9ad582647bbc3864ef78db34bdc1afdcf9b49',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ramsey/uuid',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'rhumsaa/uuid' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '4.7.0',
            ),
        ),
        'socialiteproviders/manager' => array(
            'pretty_version' => 'v4.2.0',
            'version' => '4.2.0.0',
            'reference' => '738276dfbc2b68a9145db7b3df1588d53db528a1',
            'type' => 'library',
            'install_path' => __DIR__ . '/../socialiteproviders/manager',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'spomky-labs/base64url' => array(
            'pretty_version' => 'v2.0.4',
            'version' => '2.0.4.0',
            'reference' => '7752ce931ec285da4ed1f4c5aa27e45e097be61d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../spomky-labs/base64url',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'spomky-labs/pki-framework' => array(
            'pretty_version' => '1.0.1',
            'version' => '1.0.1.0',
            'reference' => '969f3554c49ba7ed6fa603576f1f205e7d66a00c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../spomky-labs/pki-framework',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'swiftmailer/swiftmailer' => array(
            'pretty_version' => 'v6.3.0',
            'version' => '6.3.0.0',
            'reference' => '8a5d5072dca8f48460fce2f4131fcc495eec654c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../swiftmailer/swiftmailer',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/console' => array(
            'pretty_version' => 'v5.4.16',
            'version' => '5.4.16.0',
            'reference' => '8e9b9c8dfb33af6057c94e1b44846bee700dc5ef',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/console',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/css-selector' => array(
            'pretty_version' => 'v6.2.0',
            'version' => '6.2.0.0',
            'reference' => '91c342ffc99283c43653ed8eb47bc2a94db7f398',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/css-selector',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/deprecation-contracts' => array(
            'pretty_version' => 'v3.2.0',
            'version' => '3.2.0.0',
            'reference' => '1ee04c65529dea5d8744774d474e7cbd2f1206d3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/deprecation-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/error-handler' => array(
            'pretty_version' => 'v5.4.19',
            'version' => '5.4.19.0',
            'reference' => '438ef3e5e6481244785da3ce8cf8f4e74e7f2822',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/error-handler',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/event-dispatcher' => array(
            'pretty_version' => 'v6.2.5',
            'version' => '6.2.5.0',
            'reference' => 'f02d108b5e9fd4a6245aa73a9d2df2ec060c3e68',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/event-dispatcher',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/event-dispatcher-contracts' => array(
            'pretty_version' => 'v3.2.0',
            'version' => '3.2.0.0',
            'reference' => '0782b0b52a737a05b4383d0df35a474303cabdae',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/event-dispatcher-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/event-dispatcher-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '2.0|3.0',
            ),
        ),
        'symfony/finder' => array(
            'pretty_version' => 'v5.4.11',
            'version' => '5.4.11.0',
            'reference' => '7872a66f57caffa2916a584db1aa7f12adc76f8c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/finder',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/http-foundation' => array(
            'pretty_version' => 'v5.4.20',
            'version' => '5.4.20.0',
            'reference' => 'd0435363362a47c14e9cf50663cb8ffbf491875a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/http-foundation',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/http-kernel' => array(
            'pretty_version' => 'v5.4.20',
            'version' => '5.4.20.0',
            'reference' => 'aaeec341582d3c160cc9ecfa8b2419ba6c69954e',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/http-kernel',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/mime' => array(
            'pretty_version' => 'v5.4.19',
            'version' => '5.4.19.0',
            'reference' => 'a858429a9c704edc53fe057228cf9ca282ba48eb',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/mime',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-ctype' => array(
            'pretty_version' => 'v1.27.0',
            'version' => '1.27.0.0',
            'reference' => '5bbc823adecdae860bb64756d639ecfec17b050a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-ctype',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-iconv' => array(
            'pretty_version' => 'v1.27.0',
            'version' => '1.27.0.0',
            'reference' => '927013f3aac555983a5059aada98e1907d842695',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-iconv',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-intl-grapheme' => array(
            'pretty_version' => 'v1.27.0',
            'version' => '1.27.0.0',
            'reference' => '511a08c03c1960e08a883f4cffcacd219b758354',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-intl-grapheme',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-intl-idn' => array(
            'pretty_version' => 'v1.27.0',
            'version' => '1.27.0.0',
            'reference' => '639084e360537a19f9ee352433b84ce831f3d2da',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-intl-idn',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-intl-normalizer' => array(
            'pretty_version' => 'v1.27.0',
            'version' => '1.27.0.0',
            'reference' => '19bd1e4fcd5b91116f14d8533c57831ed00571b6',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-intl-normalizer',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-mbstring' => array(
            'pretty_version' => 'v1.27.0',
            'version' => '1.27.0.0',
            'reference' => '8ad114f6b39e2c98a8b0e3bd907732c207c2b534',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-mbstring',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-php72' => array(
            'pretty_version' => 'v1.27.0',
            'version' => '1.27.0.0',
            'reference' => '869329b1e9894268a8a61dabb69153029b7a8c97',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-php72',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-php73' => array(
            'pretty_version' => 'v1.27.0',
            'version' => '1.27.0.0',
            'reference' => '9e8ecb5f92152187c4799efd3c96b78ccab18ff9',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-php73',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-php80' => array(
            'pretty_version' => 'v1.27.0',
            'version' => '1.27.0.0',
            'reference' => '7a6ff3f1959bb01aefccb463a0f2cd3d3d2fd936',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-php80',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-php81' => array(
            'pretty_version' => 'v1.27.0',
            'version' => '1.27.0.0',
            'reference' => '707403074c8ea6e2edaf8794b0157a0bfa52157a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-php81',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/process' => array(
            'pretty_version' => 'v5.4.11',
            'version' => '5.4.11.0',
            'reference' => '6e75fe6874cbc7e4773d049616ab450eff537bf1',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/process',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/routing' => array(
            'pretty_version' => 'v5.4.15',
            'version' => '5.4.15.0',
            'reference' => '5c9b129efe9abce9470e384bf65d8a7e262eee69',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/routing',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/service-contracts' => array(
            'pretty_version' => 'v2.5.2',
            'version' => '2.5.2.0',
            'reference' => '4b426aac47d6427cc1a1d0f7e2ac724627f5966c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/service-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/string' => array(
            'pretty_version' => 'v6.2.2',
            'version' => '6.2.2.0',
            'reference' => '863219fd713fa41cbcd285a79723f94672faff4d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/string',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/translation' => array(
            'pretty_version' => 'v5.4.14',
            'version' => '5.4.14.0',
            'reference' => 'f0ed07675863aa6e3939df8b1bc879450b585cab',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/translation',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/translation-contracts' => array(
            'pretty_version' => 'v2.5.2',
            'version' => '2.5.2.0',
            'reference' => '136b19dd05cdf0709db6537d058bcab6dd6e2dbe',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/translation-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/translation-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '2.3',
            ),
        ),
        'symfony/var-dumper' => array(
            'pretty_version' => 'v5.4.19',
            'version' => '5.4.19.0',
            'reference' => '2944bbc23f5f8da2b962fbcbf7c4a6109b2f4b7b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/var-dumper',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/yaml' => array(
            'pretty_version' => 'v4.4.45',
            'version' => '4.4.45.0',
            'reference' => 'aeccc4dc52a9e634f1d1eebeb21eacfdcff1053d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/yaml',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'tecnickcom/tcpdf' => array(
            'pretty_version' => '6.6.2',
            'version' => '6.6.2.0',
            'reference' => 'e3cffc9bcbc76e89e167e9eb0bbda0cab7518459',
            'type' => 'library',
            'install_path' => __DIR__ . '/../tecnickcom/tcpdf',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'tightenco/ziggy' => array(
            'pretty_version' => '0.9.4',
            'version' => '0.9.4.0',
            'reference' => '82ea6ec6cb6ab3545b0245310b2a424316fe48d8',
            'type' => 'library',
            'install_path' => __DIR__ . '/../tightenco/ziggy',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'tijsverkoyen/css-to-inline-styles' => array(
            'pretty_version' => '2.2.5',
            'version' => '2.2.5.0',
            'reference' => '4348a3a06651827a27d989ad1d13efec6bb49b19',
            'type' => 'library',
            'install_path' => __DIR__ . '/../tijsverkoyen/css-to-inline-styles',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'videlalvaro/php-amqplib' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v3.4.0',
            ),
        ),
        'vlucas/phpdotenv' => array(
            'pretty_version' => 'v5.5.0',
            'version' => '5.5.0.0',
            'reference' => '1a7ea2afc49c3ee6d87061f5a233e3a035d0eae7',
            'type' => 'library',
            'install_path' => __DIR__ . '/../vlucas/phpdotenv',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'voku/portable-ascii' => array(
            'pretty_version' => '1.6.1',
            'version' => '1.6.1.0',
            'reference' => '87337c91b9dfacee02452244ee14ab3c43bc485a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../voku/portable-ascii',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'web-token/jwt-core' => array(
            'pretty_version' => '3.1.4',
            'version' => '3.1.4.0',
            'reference' => 'a1ddadec1a0e2ab605fa35e528514af8035e7371',
            'type' => 'library',
            'install_path' => __DIR__ . '/../web-token/jwt-core',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'web-token/jwt-key-mgmt' => array(
            'pretty_version' => '3.1.4',
            'version' => '3.1.4.0',
            'reference' => 'dec3ba60344bfda18e36de78fb80a2cdd8f04df5',
            'type' => 'library',
            'install_path' => __DIR__ . '/../web-token/jwt-key-mgmt',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'web-token/jwt-signature' => array(
            'pretty_version' => '3.1.4',
            'version' => '3.1.4.0',
            'reference' => 'dd51c216df7e40f3938e749254c325863dd45e14',
            'type' => 'library',
            'install_path' => __DIR__ . '/../web-token/jwt-signature',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'web-token/jwt-signature-algorithm-ecdsa' => array(
            'pretty_version' => '3.1.4',
            'version' => '3.1.4.0',
            'reference' => 'e09159600f19832cf4a68921e7299e564bc0eaf9',
            'type' => 'library',
            'install_path' => __DIR__ . '/../web-token/jwt-signature-algorithm-ecdsa',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'web-token/jwt-util-ecc' => array(
            'pretty_version' => '3.1.4',
            'version' => '3.1.4.0',
            'reference' => 'd0550dbdb2f0f652b8a45530c25ac485612be9f6',
            'type' => 'library',
            'install_path' => __DIR__ . '/../web-token/jwt-util-ecc',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'webmozart/assert' => array(
            'pretty_version' => '1.11.0',
            'version' => '1.11.0.0',
            'reference' => '11cb2199493b2f8a3b53e7f19068fc6aac760991',
            'type' => 'library',
            'install_path' => __DIR__ . '/../webmozart/assert',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
    ),
);
