<?php

return [

    /*
    |--------------------------------------------------------------------------
    | Sesnors Language Lines
    |--------------------------------------------------------------------------
    |
    | The following language lines are used to translate names and units of sensors
    |
    */

    'title' => 'Сенсори',
    'airflow' => [
        'short' => 'Повітряний потік',
        'long' => 'Повітряний потік',
        'unit' => 'cfm',
        'unit_long' => 'Кубічний фут за хвилину',
    ],
    'ber' => [
        'short' => 'BER',
        'long' => 'Частота бітових помилок',
        'unit' => '',
        'unit_long' => '',
    ],
    'charge' => [
        'short' => 'Заряд',
        'long' => 'Процент заряду',
        'unit' => '%',
        'unit_long' => 'процентів',
    ],
    'chromatic_dispersion' => [
        'short' => 'Хроматична дисперсія',
        'long' => 'Хроматична дисперсія',
        'unit' => 'ps/nm/km',
        'unit_long' => 'Пікосекунди на нанометри на кілометри',
    ],
    'cooling' => [
        'short' => 'Охолодження',
        'long' => '',
        'unit' => 'W',
        'unit_long' => 'Ват',
    ],
    'count' => [
        'short' => 'Лічильник',
        'long' => 'Лічильник',
        'unit' => '',
        'unit_long' => '',
    ],
    'current' => [
        'short' => 'Сила струму',
        'long' => 'Сила струму',
        'unit' => 'A',
        'unit_long' => 'Ампер',
    ],
    'dbm' => [
        'short' => 'dBm',
        'long' => 'dBm',
        'unit' => 'dBm',
        'unit_long' => 'Децибел-мілівати',
    ],
    'delay' => [
        'short' => 'Затримка',
        'long' => 'Затримка',
        'unit' => 's',
        'unit_long' => 'Секунди',
    ],
    'eer' => [
        'short' => 'EER',
        'long' => 'Коефіцієнт енергоефективності',
        'unit' => '',
        'unit_long' => '',
    ],
    'fanspeed' => [
        'short' => 'Оберти вентилятора',
        'long' => 'Швидкість обертів вентилятора',
        'unit' => 'RPM',
        'unit_long' => 'Обертів за хвилину',
    ],
    'frequency' => [
        'short' => 'Частота',
        'long' => 'Частота',
        'unit' => 'Hz',
        'unit_long' => 'Герц',
    ],
    'humidity' => [
        'short' => 'Вологість',
        'long' => 'Процент вологості повітря',
        'unit' => '%',
        'unit_long' => 'Проценти',
    ],
    'load' => [
        'short' => 'Навантаження',
        'long' => 'Процент навантаження',
        'unit' => '%',
        'unit_long' => 'Проценти',
    ],
    'loss' => [
        'short' => 'Процент',
        'long' => 'Процент втрати',
        'unit' => '%',
        'unit_long' => 'процентів',
    ],
    'power' => [
        'short' => 'Потужність',
        'long' => 'Потужність',
        'unit' => 'W',
        'unit_long' => 'Ват',
    ],
    'power_consumed' => [
        'short' => 'Спожита потужність',
        'long' => 'Спожита потужність',
        'unit' => 'kWh',
        'unit_long' => 'Кіловат-години',
    ],
    'power_factor' => [
        'short' => 'Коефіцієнт потужності',
        'long' => 'Коефіцієнт потужності',
        'unit' => '',
        'unit_long' => '',
    ],
    'pressure' => [
        'short' => 'Тиск',
        'long' => 'Тиск',
        'unit' => 'kPa',
        'unit_long' => 'Кілопаскалі',
    ],
    'quality_factor' => [
        'short' => 'Добротність',
        'long' => 'Добротність',
        'unit' => '',
        'unit_long' => '',
    ],
    'runtime' => [
        'short' => 'Час роботи',
        'long' => 'Час роботи',
        'unit' => 'хвилини',
        'unit_long' => 'Хвилини',
    ],
    'signal' => [
        'short' => 'Сигнал',
        'long' => 'Сигнал',
        'unit' => 'dBm',
        'unit_long' => 'Децибел-мілівати',
    ],
    'tv_signal' => [
        'short' => 'Сигнал',
        'long' => 'Телесигнал',
        'unit' => 'dBmV',
        'unit_long' => 'децибел-мілівольти',
    ],
    'bitrate' => [
        'short' => 'Бітрейт',
        'long' => 'Бітрейт',
        'unit' => 'bps',
        'unit_long' => 'біти за секунду',
    ],
    'snr' => [
        'short' => 'SNR',
        'long' => 'Відношення сигналу до шуму',
        'unit' => 'dB',
        'unit_long' => 'Децибели',
    ],
    'state' => [
        'short' => 'Стан',
        'long' => 'Стан',
        'unit' => '',
    ],
    'temperature' => [
        'short' => 'Температура',
        'long' => 'Температура',
        'unit' => '°C',
        'unit_long' => '° Цельсія',
    ],
    'voltage' => [
        'short' => 'Напруга',
        'long' => 'Напруга',
        'unit' => 'V',
        'unit_long' => 'Вольт',
    ],
    'waterflow' => [
        'short' => 'Потік води',
        'long' => 'Потік води',
        'unit' => 'l/m',
        'unit_long' => 'Літри за хвилину',
    ],
    'percent' => [
        'short' => 'Процент',
        'long' => 'Процент',
        'unit' => '%',
        'unit_long' => 'Процент',
    ],
];
