<?php

return [
    'settings_page' => ':plugin: Налаштування',
    'admin_page' => 'Управління плагінами',
    'admin_title' => 'Плагіни',
    'errors' => [
        'not_exist' => 'Плагін :plugin не існує.',
        'disabled' => 'Плагін :plugin вимкнено.',
        'view_missing' => 'Немає виду.',
    ],
];
