<?php

return [

    /*
    |--------------------------------------------------------------------------
    | Wireless Sensors Language Lines
    |--------------------------------------------------------------------------
    |
    | The following language lines are used to translate names and units of wireless sensors
    |
    */

    'title' => 'Бездротові',
    'ap-count' => [
        'short' => 'AP',
        'long' => 'Кількість точок доступу',
        'unit' => '',
    ],
    'clients' => [
        'short' => 'Споживачі',
        'long' => 'Кількість споживачів',
        'unit' => '',
    ],
    'capacity' => [
        'short' => 'Ємність',
        'long' => 'Ємність',
        'unit' => '%',
    ],
    'ccq' => [
        'short' => 'CCQ',
        'long' => "Якість з'єднання споживача",
        'unit' => '%',
    ],
    'errors' => [
        'short' => 'Помилки',
        'long' => 'Кількість помилок',
        'unit' => '',
    ],
    'error-ratio' => [
        'short' => 'Відсоток помилок',
        'long' => 'Відсоток помилкових бітів/пакетів',
        'unit' => '%',
    ],
    'error-rate' => [
        'short' => 'BER',
        'long' => 'Частота бітових помилок',
        'unit' => 'bps',
    ],
    'frequency' => [
        'short' => 'Частота',
        'long' => 'Частота',
        'unit' => 'MHz',
    ],
    'distance' => [
        'short' => 'Відстань',
        'long' => 'Відстань',
        'unit' => 'm',
    ],
    'mse' => [
        'short' => 'MSE',
        'long' => 'Середньоквадратична помилка',
        'unit' => 'dB',
    ],
    'noise-floor' => [
        'short' => 'Рівень шуму',
        'long' => 'Рівень шуму',
        'unit' => 'dBm',
    ],
    'power' => [
        'short' => 'Сила сигналу',
        'long' => 'TX/RX Сила сигналу',
        'unit' => 'dBm',
    ],
    'quality' => [
        'short' => 'Якість',
        'long' => 'Якість',
        'unit' => '%',
    ],
    'rate' => [
        'short' => 'Швидкість передачі даних',
        'long' => 'Швидкість передачі даних TX/RX',
        'unit' => 'bps',
    ],
    'rssi' => [
        'short' => 'RSSI',
        'long' => 'Індикатор потужності отриманого сигналу',
        'unit' => 'dBm',
    ],
    'snr' => [
        'short' => 'SNR',
        'long' => 'Співвідношення сигнал/шум',
        'unit' => 'dB',
    ],
    'sinr' => [
        'short' => 'SINR',
        'long' => ' Співвідношення сигнал/завади+шум',
        'unit' => 'dB',
    ],
    'rsrq' => [
        'short' => 'RSRQ',
        'long' => 'Якість отриманого еталонного сигналу',
        'unit' => 'dB',
    ],
    'rsrp' => [
        'short' => 'RSRP',
        'long' => 'Сила отриманого еталонного сигналу',
        'unit' => 'dBm',
    ],
    'ssr' => [
        'short' => 'SSR',
        'long' => 'Коефіцієнт потужності сигналу',
        'unit' => 'dB',
    ],
    'utilization' => [
        'short' => 'Використання',
        'long' => 'Використання',
        'unit' => '%',
    ],
    'xpi' => [
        'short' => 'XPI',
        'long' => 'Крос-поляризаційні завади',
        'unit' => 'dB',
    ],
    'cell' => [
        'short' => 'Cell',
        'long' => 'Cell',
        'unit' => '',
    ],
    'channel' => [
        'short' => 'Канал',
        'long' => 'Канал',
        'unit' => '',
    ],
];
