<?php

/**
 * Afrikaans PHPMailer language file: refer to English translation for definitive list
 * @package PHPMailer
 */

$PHPMAILER_LANG['authenticate']         = 'SMTP-fout: kon nie geverifieer word nie.';
$PHPMAILER_LANG['connect_host']         = 'SMTP-fout: kon nie aan SMTP-verbind nie.';
$PHPMAILER_LANG['data_not_accepted']    = 'SMTP-fout: data nie aanvaar nie.';
$PHPMAILER_LANG['empty_message']        = 'Boodskapliggaam leeg.';
$PHPMAILER_LANG['encoding']             = 'Onbekende kodering: ';
$PHPMAILER_LANG['execute']              = 'Kon nie uitvoer nie: ';
$PHPMAILER_LANG['file_access']          = 'Kon nie lêer oopmaak nie: ';
$PHPMAILER_LANG['file_open']            = 'Lêerfout: Kon nie lêer oopmaak nie: ';
$PHPMAILER_LANG['from_failed']          = 'Die volgende Van adres misluk: ';
$PHPMAILER_LANG['instantiate']          = 'Kon nie posfunksie instansieer nie.';
$PHPMAILER_LANG['invalid_address']      = 'Ongeldige adres: ';
$PHPMAILER_LANG['mailer_not_supported'] = ' mailer word nie ondersteun nie.';
$PHPMAILER_LANG['provide_address']      = 'U moet ten minste een ontvanger e-pos adres verskaf.';
$PHPMAILER_LANG['recipients_failed']    = 'SMTP-fout: Die volgende ontvangers het misluk: ';
$PHPMAILER_LANG['signing']              = 'Ondertekening Fout: ';
$PHPMAILER_LANG['smtp_connect_failed']  = 'SMTP-verbinding () misluk.';
$PHPMAILER_LANG['smtp_error']           = 'SMTP-bediener fout: ';
$PHPMAILER_LANG['variable_set']         = 'Kan nie veranderlike instel of herstel nie: ';
$PHPMAILER_LANG['extension_missing']    = 'Uitbreiding ontbreek: ';
