CREATE TABLE IF NOT EXISTS `munin_plugins` (  `mplug_id` int(11) NOT NULL AUTO_INCREMENT,  `device_id` int(11) NOT NULL,  `mplug_type` varchar(256) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,  `mplug_instance` varchar(128) CHARACTER SET utf8 COLLATE utf8_bin DEFAULT NULL,  `mplug_category` varchar(32) CHARACTER SET utf8 COLLATE utf8_bin DEFAULT NULL,  `mplug_title` varchar(128) CHARACTER SET utf8 COLLATE utf8_bin DEFAULT NULL,  `mplug_info` text CHARACTER SET utf8 COLLATE utf8_bin,  `mplug_vlabel` varchar(128) CHARACTER SET utf8 COLLATE utf8_bin DEFAULT NULL,  `mplug_args` varchar(512) CHARACTER SET utf8 COLLATE utf8_bin DEFAULT NULL,  `mplug_total` binary(1) NOT NULL DEFAULT '0',  `mplug_graph` binary(1) NOT NULL DEFAULT '1',  PRIMARY KEY (`mplug_id`),  UNIQUE KEY `UNIQUE` (`device_id`,`mplug_type`),  KEY `device_id` (`device_id`)) ENGINE=MyISAM  DEFAULT CHARSET=utf8  COLLATE=utf8_bin ;
CREATE TABLE IF NOT EXISTS `munin_plugins_ds` (  `mplug_id` int(11) NOT NULL,  `ds_name` varchar(32) COLLATE utf8_bin NOT NULL,  `ds_type` enum('COUNTER','ABSOLUTE','DERIVE','GAUGE') COLLATE utf8_bin NOT NULL DEFAULT 'GAUGE',  `ds_label` varchar(64) COLLATE utf8_bin NOT NULL,  `ds_cdef` text COLLATE utf8_bin NOT NULL,  `ds_draw` varchar(64) COLLATE utf8_bin NOT NULL,  `ds_graph` enum('no','yes') COLLATE utf8_bin NOT NULL DEFAULT 'yes',  `ds_info` varchar(255) COLLATE utf8_bin NOT NULL,  `ds_extinfo` text COLLATE utf8_bin NOT NULL,  `ds_max` varchar(32) COLLATE utf8_bin NOT NULL,  `ds_min` varchar(32) COLLATE utf8_bin NOT NULL,  `ds_negative` varchar(32) COLLATE utf8_bin NOT NULL,  `ds_warning` varchar(32) COLLATE utf8_bin NOT NULL,  `ds_critical` varchar(32) COLLATE utf8_bin NOT NULL,  `ds_colour` varchar(32) COLLATE utf8_bin NOT NULL,  `ds_sum` text CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,  `ds_stack` text COLLATE utf8_bin NOT NULL,  `ds_line` varchar(64) COLLATE utf8_bin NOT NULL,  UNIQUE KEY `splug_id` (`mplug_id`,`ds_name`)) ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_bin;
