<?php

return [
    'notification-subscription-status' => [
        'no-support' => 'Ovaj pregledać ne podržava forsirana obaveštenja',
        'no-transport' => 'Mora postojati transport alarma koji je vezan za ovog korisnika kako bi omogućili forsirana obaveštenja',
        'enabled' => 'Forsirana obaveštenje su omogućena za ovaj pretraživač',
        'disabled' => 'Forsirana obaveštenja su onemogućena za ovaj pretraživač',
        'enable' => 'Omogući',
        'disable' => 'Onemogući',
    ],
];
