<?php

return [
    'settings_page' => 'Impostazioni plugin :plugin',
    'admin_page' => 'Gestione plugin',
    'admin_title' => 'Plugin',
    'errors' => [
        'not_exist' => 'Il plugin :plugin non esiste.',
        'disabled' => 'Plugin :plugin disabilitato.',
        'view_missing' => 'Mostra quelli mancanti.',
    ],
];
