<?php

return [
    'attributes' => [
        'features' => 'Властивості OS',
        'hardware' => 'Апаратне забезпечення',
        'icon' => 'Іконка',
        'location' => 'Місцезнаходження',
        'os' => 'OS пристрою',
        'serial' => 'Серійний номер',
        'sysName' => 'sysName',
        'version' => 'Версія OS',
        'type' => 'Тип пристрою',
    ],
];
