<?php

return [

    /*
    |--------------------------------------------------------------------------
    | Password Reset Language Lines
    |--------------------------------------------------------------------------
    |
    | The following language lines are the default lines which match reasons
    | that are given by the password broker for a password update attempt
    | has failed, such as for an invalid token or invalid new password.
    |
    */

    'password' => 'Пароль должен содержать не менее шести символов и должен совпадать с подтверждением.',
    'reset' => 'Ваш пароль был сброшен!',
    'sent' => 'Мы отправили вам ссылку на email для обновления пароля',
    'token' => 'Токен обновления пароля валиден.',
    'user' => 'Мы не можем найти пользователя с таким адресом email.',

];
