INSERT INTO config (config_name,config_value,config_default,config_descr,config_group,config_group_order,config_sub_group,config_sub_group_order,config_hidden,config_disabled) values ('peeringdb.enabled','false','false','Enable PeeringDB lookup (data is downloaded with daily.sh)','external',0,'peeringdb',0,'0','0');
CREATE TABLE `pdb_ix` (  `pdb_ix_id` int(10) unsigned NOT NULL AUTO_INCREMENT,  `ix_id` int(10) unsigned NOT NULL,  `name` varchar(255) COLLATE utf8_unicode_ci NOT NULL,  `asn` int(10) unsigned NOT NULL,  `timestamp` int(10) unsigned NOT NULL,  PRIMARY KEY (`pdb_ix_id`)) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;
CREATE TABLE `pdb_ix_peers` (  `pdb_ix_peers_id` int(10) unsigned NOT NULL AUTO_INCREMENT,  `ix_id` int(10) unsigned NOT NULL,  `peer_id` int(10) unsigned NOT NULL,  `remote_asn` varchar(16) COLLATE utf8_unicode_ci NOT NULL, `remote_ipaddr4` VARCHAR(15) COLLATE utf8_unicode_ci NULL, `remote_ipaddr6` VARCHAR(128) COLLATE utf8_unicode_ci NULL,  `name` varchar(255) COLLATE utf8_unicode_ci DEFAULT NULL,  `timestamp` int(10) unsigned DEFAULT NULL,  PRIMARY KEY (`pdb_ix_peers_id`)) ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;
