<?php

return [
    'attributes' => [
        'features' => 'OS Features',
        'hardware' => 'Hardware',
        'icon' => 'Icona',
        'location' => 'Location',
        'os' => 'S.O. del dispositivo',
        'serial' => 'Seriale',
        'sysName' => 'sysName',
        'version' => 'Versione del S.O.',
        'type' => 'Tipo dispositivo',
    ],
];
