ALTER TABLE `ports` MODIFY `ifInUcastPkts` BIGINT UNSIGNED, MODIFY `ifInUcastPkts_prev` BIGINT UNSIGNED, MODIFY `ifInUcastPkts_delta` BIGINT UNSIGNED, MODIFY `ifInUcastPkts_rate` BIGINT UNSIGNED, MODIFY `ifOutUcastPkts` BIGINT UNSIGNED, MODIFY `ifOutUcastPkts_prev` BIGINT UNSIGNED, MODIFY `ifOutUcastPkts_delta` BIGINT UNSIGNED, MODIFY `ifOutUcastPkts_rate` BIGINT UNSIGNED, MODIFY `ifInErrors` BIGINT UNSIGNED, MODIFY `ifInErrors_prev` BIGINT UNSIGNED, MODIFY `ifInErrors_delta` BIGINT UNSIGNED, MODIFY `ifInErrors_rate` BIGINT UNSIGNED, MODIFY `ifOutErrors` BIGINT UNSIGNED, MODIFY `ifOutErrors_prev` BIGINT UNSIGNED, MODIFY `ifOutErrors_delta` BIGINT UNSIGNED, MODIFY `ifOutErrors_rate` BIGINT UNSIGNED, MODIFY `ifInOctets` BIGINT UNSIGNED, MODIFY `ifInOctets_prev` BIGINT UNSIGNED, MODIFY `ifInOctets_delta` BIGINT UNSIGNED, MODIFY `ifInOctets_rate` BIGINT UNSIGNED, MODIFY `ifOutOctets` BIGINT UNSIGNED, MODIFY `ifOutOctets_prev` BIGINT UNSIGNED, MODIFY `ifOutOctets_delta` BIGINT UNSIGNED, MODIFY `ifOutOctets_rate` BIGINT UNSIGNED;
