<?php

$graph_type = 'sensor_temperature';
$sensor_class = 'temperature';
$sensor_unit = '&deg;C';
$sensor_type = 'Temperature';

require 'includes/html/pages/device/overview/generic/sensor.inc.php';
